@@ -1,7 +1,5 @@
-# -*- fill-column: 78 -*-
 # mysql/base.py
-# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
-# and Jason Kirtland.
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -41,23 +39,6 @@ Connecting
 ----------
 
 See the API documentation on individual drivers for details on connecting.
-    
-Data Types
-----------
-
-All of MySQL's standard types are supported.   These can also be specified within 
-table metadata, for the purpose of issuing CREATE TABLE statements
-which include MySQL-specific extensions.   The types are available
-from the module, as in::
-
-    from sqlalchemy.dialects import mysql
-
-    Table('mytable', metadata,
-          Column('id', Integer, primary_key=True),
-          Column('ittybittyblob', mysql.TINYBLOB),
-          Column('biggy', mysql.BIGINT(unsigned=True)))
-
-See the API documentation on specific column types for further details.
 
 Connection Timeouts
 -------------------
@@ -87,6 +68,22 @@ creation option can be specified in this syntax::
         mysql_charset='utf8'
        )
 
+Case Sensitivity and Table Reflection
+-------------------------------------
+
+MySQL has inconsistent support for case-sensitive identifier
+names, basing support on specific details of the underlying
+operating system. However, it has been observed that no matter
+what case sensitivity behavior is present, the names of tables in
+foreign key declarations are *always* received from the database
+as all-lower case, making it impossible to accurately reflect a
+schema where inter-related tables use mixed-case identifier names.
+
+Therefore it is strongly advised that table names be declared as
+all lower case both within SQLAlchemy as well as on the MySQL
+database itself, especially if database reflection features are
+to be used.
+
 Keys
 ----
 
@@ -100,7 +97,7 @@ foreign keys.  For these tables, you may supply a
         autoload=True
        )
 
-When creating tables, SQLAlchemy will automatically set ``AUTO_INCREMENT``` on
+When creating tables, SQLAlchemy will automatically set ``AUTO_INCREMENT`` on
 an integer primary key column::
 
   >>> t = Table('mytable', metadata,
@@ -171,14 +168,6 @@ available.
 
     update(..., mysql_limit=10)
 
-Troubleshooting
----------------
-
-If you have problems that seem server related, first check that you are
-using the most recent stable MySQL-Python package available.  The Database
-Notes page on the wiki at http://www.sqlalchemy.org is a good resource for
-timely information affecting MySQL in SQLAlchemy.
-
 """
 
 import datetime, inspect, re, sys
@@ -250,11 +239,11 @@ SET_RE = re.compile(
 class _NumericType(object):
     """Base for MySQL numeric types."""
 
-    def __init__(self, **kw):
-        self.unsigned = kw.pop('unsigned', False)
-        self.zerofill = kw.pop('zerofill', False)
+    def __init__(self, unsigned=False, zerofill=False, **kw):
+        self.unsigned = unsigned
+        self.zerofill = zerofill
         super(_NumericType, self).__init__(**kw)
-        
+
 class _FloatType(_NumericType, sqltypes.Float):
     def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
         if isinstance(self, (REAL, DOUBLE)) and \
@@ -293,7 +282,7 @@ class _StringType(sqltypes.String):
         self.binary = binary
         self.national = national
         super(_StringType, self).__init__(**kw)
-        
+
     def __repr__(self):
         attributes = inspect.getargspec(self.__init__)[0][1:]
         attributes.extend(inspect.getargspec(_StringType.__init__)[0][1:])
@@ -310,9 +299,9 @@ class _StringType(sqltypes.String):
 
 class NUMERIC(_NumericType, sqltypes.NUMERIC):
     """MySQL NUMERIC type."""
-    
+
     __visit_name__ = 'NUMERIC'
-    
+
     def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
         """Construct a NUMERIC.
 
@@ -334,9 +323,9 @@ class NUMERIC(_NumericType, sqltypes.NUMERIC):
 
 class DECIMAL(_NumericType, sqltypes.DECIMAL):
     """MySQL DECIMAL type."""
-    
+
     __visit_name__ = 'DECIMAL'
-    
+
     def __init__(self, precision=None, scale=None, asdecimal=True, **kw):
         """Construct a DECIMAL.
 
@@ -356,7 +345,7 @@ class DECIMAL(_NumericType, sqltypes.DECIMAL):
         super(DECIMAL, self).__init__(precision=precision, scale=scale,
                                       asdecimal=asdecimal, **kw)
 
-    
+
 class DOUBLE(_FloatType):
     """MySQL DOUBLE type."""
 
@@ -381,7 +370,7 @@ class DOUBLE(_FloatType):
         super(DOUBLE, self).__init__(precision=precision, scale=scale,
                                      asdecimal=asdecimal, **kw)
 
-class REAL(_FloatType):
+class REAL(_FloatType, sqltypes.REAL):
     """MySQL REAL type."""
 
     __visit_name__ = 'REAL'
@@ -557,12 +546,12 @@ class BIT(sqltypes.TypeEngine):
 
     def result_processor(self, dialect, coltype):
         """Convert a MySQL's 64 bit, variable length binary string to a long.
-        
+
         TODO: this is MySQL-db, pyodbc specific.  OurSQL and mysqlconnector
         already do this, so this logic should be moved to those dialects.
-        
+
         """
-        
+
         def process(value):
             if value is not None:
                 v = 0L
@@ -729,7 +718,7 @@ class LONGTEXT(_StringType):
         """
         super(LONGTEXT, self).__init__(**kwargs)
 
-    
+
 class VARCHAR(_StringType, sqltypes.VARCHAR):
     """MySQL VARCHAR type, for variable-length character data."""
 
@@ -766,7 +755,7 @@ class CHAR(_StringType, sqltypes.CHAR):
 
     __visit_name__ = 'CHAR'
 
-    def __init__(self, length, **kwargs):
+    def __init__(self, length=None, **kwargs):
         """Construct a CHAR.
 
         :param length: Maximum data length, in characters.
@@ -817,7 +806,7 @@ class NCHAR(_StringType, sqltypes.NCHAR):
     __visit_name__ = 'NCHAR'
 
     def __init__(self, length=None, **kwargs):
-        """Construct an NCHAR.  Arguments are:
+        """Construct an NCHAR.
 
         :param length: Maximum data length, in characters.
 
@@ -837,7 +826,7 @@ class NCHAR(_StringType, sqltypes.NCHAR):
 
 class TINYBLOB(sqltypes._Binary):
     """MySQL TINYBLOB type, for binary data up to 2^8 bytes."""
-    
+
     __visit_name__ = 'TINYBLOB'
 
 class MEDIUMBLOB(sqltypes._Binary):
@@ -862,8 +851,6 @@ class ENUM(sqltypes.Enum, _StringType):
 
           Column('myenum', MSEnum("foo", "bar", "baz"))
 
-        Arguments are:
-
         :param enums: The range of valid values for this ENUM.  Values will be
           quoted when generating the schema according to the quoting flag (see
           below).
@@ -907,7 +894,7 @@ class ENUM(sqltypes.Enum, _StringType):
 
         """
         self.quoting = kw.pop('quoting', 'auto')
-        
+
         if self.quoting == 'auto' and len(enums):
             # What quoting character are we using?
             q = None
@@ -940,7 +927,7 @@ class ENUM(sqltypes.Enum, _StringType):
         kw.pop('native_enum', None)
         _StringType.__init__(self, length=length, **kw)
         sqltypes.Enum.__init__(self, *enums)
-    
+
     @classmethod
     def _strip_enums(cls, enums):
         strip_enums = []
@@ -950,7 +937,7 @@ class ENUM(sqltypes.Enum, _StringType):
                 a = a[1:-1].replace(a[0] * 2, a[0])
             strip_enums.append(a)
         return strip_enums
-        
+
     def bind_processor(self, dialect):
         super_convert = super(ENUM, self).bind_processor(dialect)
         def process(value):
@@ -963,6 +950,10 @@ class ENUM(sqltypes.Enum, _StringType):
                 return value
         return process
 
+    def adapt(self, impltype, **kw):
+        kw['strict'] = self.strict
+        return sqltypes.Enum.adapt(self, impltype, **kw)
+
 class SET(_StringType):
     """MySQL SET type."""
 
@@ -975,8 +966,6 @@ class SET(_StringType):
 
           Column('myset', MSSet("'foo'", "'bar'", "'baz'"))
 
-        Arguments are:
-
         :param values: The range of valid values for this SET.  Values will be
           used exactly as they appear when generating schemas.  Strings must
           be quoted, as in the example above.  Single-quotes are suggested for
@@ -1011,8 +1000,8 @@ class SET(_StringType):
             strip_values.append(a)
 
         self.values = strip_values
-        length = max([len(v) for v in strip_values] + [0])
-        super(SET, self).__init__(length=length, **kw)
+        kw.setdefault('length', max([len(v) for v in strip_values] + [0]))
+        super(SET, self).__init__(**kw)
 
     def result_processor(self, dialect, coltype):
         def process(value):
@@ -1140,19 +1129,19 @@ class MySQLCompiler(compiler.SQLCompiler):
     extract_map.update ({
         'milliseconds': 'millisecond',
     })
-    
+
     def visit_random_func(self, fn, **kw):
         return "rand%s" % self.function_argspec(fn)
-    
+
     def visit_utc_timestamp_func(self, fn, **kw):
         return "UTC_TIMESTAMP"
-    
+
     def visit_sysdate_func(self, fn, **kw):
         return "SYSDATE()"
-        
+
     def visit_concat_op(self, binary, **kw):
         return "concat(%s, %s)" % (self.process(binary.left), self.process(binary.right))
-        
+
     def visit_match_op(self, binary, **kw):
         return "MATCH (%s) AGAINST (%s IN BOOLEAN MODE)" % (self.process(binary.left), self.process(binary.right))
 
@@ -1180,7 +1169,7 @@ class MySQLCompiler(compiler.SQLCompiler):
                 return 'CHAR'
         elif isinstance(type_, sqltypes._Binary):
             return 'BINARY'
-        elif isinstance(type_, NUMERIC):
+        elif isinstance(type_, sqltypes.NUMERIC):
             return self.dialect.type_compiler.process(type_).replace('NUMERIC', 'DECIMAL')
         else:
             return None
@@ -1189,7 +1178,7 @@ class MySQLCompiler(compiler.SQLCompiler):
         # No cast until 4, no decimals until 5.
         if not self.dialect._supports_cast:
             return self.process(cast.clause)
-        
+
         type_ = self.process(cast.typeclause)
         if type_ is None:
             return self.process(cast.clause)
@@ -1201,8 +1190,15 @@ class MySQLCompiler(compiler.SQLCompiler):
         if self.dialect._backslash_escapes:
             value = value.replace('\\', '\\\\')
         return value
-        
+
     def get_select_precolumns(self, select):
+        """Add special MySQL keywords in place of DISTINCT.
+        
+        .. note:: this usage is deprecated.  :meth:`.Select.prefix_with`
+          should be used for special keywords at the start
+          of a SELECT.
+          
+        """
         if isinstance(select._distinct, basestring):
             return select._distinct.upper() + " "
         elif select._distinct:
@@ -1245,12 +1241,24 @@ class MySQLCompiler(compiler.SQLCompiler):
         elif offset is not None:
             # As suggested by the MySQL docs, need to apply an
             # artificial limit if one wasn't provided
+            # http://dev.mysql.com/doc/refman/5.0/en/select.html
             if limit is None:
-                limit = 18446744073709551615
-            return ' \n LIMIT %s, %s' % (offset, limit)
+                # hardwire the upper limit.  Currently
+                # needed by OurSQL with Python 3
+                # (https://bugs.launchpad.net/oursql/+bug/686232), 
+                # but also is consistent with the usage of the upper
+                # bound as part of MySQL's "syntax" for OFFSET with
+                # no LIMIT
+                return ' \n LIMIT %s, %s' % (
+                                self.process(sql.literal(offset)), 
+                                "18446744073709551615")
+            else:
+                return ' \n LIMIT %s, %s' % (
+                                self.process(sql.literal(offset)), 
+                                self.process(sql.literal(limit)))
         else:
             # No offset provided, so just use the limit
-            return ' \n LIMIT %s' % (limit,)
+            return ' \n LIMIT %s' % (self.process(sql.literal(limit)),)
 
     def visit_update(self, update_stmt):
         self.stack.append({'from': set([update_stmt.table])})
@@ -1264,7 +1272,7 @@ class MySQLCompiler(compiler.SQLCompiler):
         if update_stmt._whereclause is not None:
             text += " WHERE " + self.process(update_stmt._whereclause)
 
-        limit = update_stmt.kwargs.get('mysql_limit', None)
+        limit = update_stmt.kwargs.get('%s_limit' % self.dialect.name, None)
         if limit:
             text += " LIMIT %s" % limit
 
@@ -1281,9 +1289,10 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
     def create_table_constraints(self, table):
         """Get table constraints."""
         constraint_string = super(MySQLDDLCompiler, self).create_table_constraints(table)
-        
-        is_innodb = table.kwargs.has_key('mysql_engine') and \
-                    table.kwargs['mysql_engine'].lower() == 'innodb'
+
+        engine_key = '%s_engine' % self.dialect.name
+        is_innodb = table.kwargs.has_key(engine_key) and \
+                    table.kwargs[engine_key].lower() == 'innodb'
 
         auto_inc_column = table._autoincrement_column
 
@@ -1294,7 +1303,7 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
                 constraint_string += ", \n\t"
             constraint_string += "KEY `idx_autoinc_%s`(`%s`)" % (auto_inc_column.name, \
                             self.preparer.format_column(auto_inc_column))
-        
+
         return constraint_string
 
 
@@ -1308,7 +1317,7 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
         default = self.get_column_default_string(column)
         if default is not None:
             colspec.append('DEFAULT ' + default)
-        
+
         is_timestamp = isinstance(column.type, sqltypes.TIMESTAMP)
         if not column.nullable and not is_timestamp:
             colspec.append('NOT NULL')
@@ -1316,16 +1325,8 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
         elif column.nullable and is_timestamp and default is None:
             colspec.append('NULL')
 
-        if column.primary_key and column.autoincrement:
-            try:
-                first = [c for c in column.table.primary_key.columns
-                         if (c.autoincrement and
-                             isinstance(c.type, sqltypes.Integer) and
-                             not c.foreign_keys)].pop(0)
-                if column is first:
-                    colspec.append('AUTO_INCREMENT')
-            except IndexError:
-                pass
+        if column is column.table._autoincrement_column and column.server_default is None:
+            colspec.append('AUTO_INCREMENT')
 
         return ' '.join(colspec)
 
@@ -1334,15 +1335,16 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
 
         table_opts = []
         for k in table.kwargs:
-            if k.startswith('mysql_'):
-                opt = k[6:].upper()
+            if k.startswith('%s_' % self.dialect.name):
+                opt = k[len(self.dialect.name)+1:].upper()
 
                 arg = table.kwargs[k]
                 if opt in _options_of_type_string:
                     arg = "'%s'" % arg.replace("\\", "\\\\").replace("'", "''")
 
                 if opt in ('DATA_DIRECTORY', 'INDEX_DIRECTORY',
-                           'DEFAULT_CHARACTER_SET', 'CHARACTER_SET', 'DEFAULT_CHARSET',
+                           'DEFAULT_CHARACTER_SET', 'CHARACTER_SET', 
+                           'DEFAULT_CHARSET',
                            'DEFAULT_COLLATE'):
                     opt = opt.replace('_', ' ')
 
@@ -1356,9 +1358,9 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
 
     def visit_drop_index(self, drop):
         index = drop.element
-        
+
         return "\nDROP INDEX %s ON %s" % \
-                    (self.preparer.quote(self._validate_identifier(index.name, False), index.quote),
+                    (self.preparer.quote(self._index_identifier(index.name), index.quote),
                      self.preparer.format_table(index.table))
 
     def visit_drop_constraint(self, drop):
@@ -1397,10 +1399,10 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
         COLLATE annotations and MySQL specific extensions.
 
         """
-        
+
         def attr(name):
             return getattr(type_, name, defaults.get(name))
-            
+
         if attr('charset'):
             charset = 'CHARACTER SET %s' % attr('charset')
         elif attr('ascii'):
@@ -1423,25 +1425,33 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
                              if c is not None])
         return ' '.join([c for c in (spec, charset, collation)
                          if c is not None])
-    
+
     def _mysql_type(self, type_):
         return isinstance(type_, (_StringType, _NumericType))
-    
+
     def visit_NUMERIC(self, type_):
         if type_.precision is None:
             return self._extend_numeric(type_, "NUMERIC")
         elif type_.scale is None:
-            return self._extend_numeric(type_, "NUMERIC(%(precision)s)" % {'precision': type_.precision})
+            return self._extend_numeric(type_, 
+                            "NUMERIC(%(precision)s)" % 
+                            {'precision': type_.precision})
         else:
-            return self._extend_numeric(type_, "NUMERIC(%(precision)s, %(scale)s)" % {'precision': type_.precision, 'scale' : type_.scale})
+            return self._extend_numeric(type_, 
+                            "NUMERIC(%(precision)s, %(scale)s)" % 
+                            {'precision': type_.precision, 'scale' : type_.scale})
 
     def visit_DECIMAL(self, type_):
         if type_.precision is None:
             return self._extend_numeric(type_, "DECIMAL")
         elif type_.scale is None:
-            return self._extend_numeric(type_, "DECIMAL(%(precision)s)" % {'precision': type_.precision})
+            return self._extend_numeric(type_, 
+                            "DECIMAL(%(precision)s)" % 
+                            {'precision': type_.precision})
         else:
-            return self._extend_numeric(type_, "DECIMAL(%(precision)s, %(scale)s)" % {'precision': type_.precision, 'scale' : type_.scale})
+            return self._extend_numeric(type_, 
+                            "DECIMAL(%(precision)s, %(scale)s)" % 
+                            {'precision': type_.precision, 'scale' : type_.scale})
 
     def visit_DOUBLE(self, type_):
         if type_.precision is not None and type_.scale is not None:
@@ -1458,30 +1468,39 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
                                  'scale' : type_.scale})
         else:
             return self._extend_numeric(type_, 'REAL')
-    
+
     def visit_FLOAT(self, type_):
-        if self._mysql_type(type_) and type_.scale is not None and type_.precision is not None:
-            return self._extend_numeric(type_, "FLOAT(%s, %s)" % (type_.precision, type_.scale))
+        if self._mysql_type(type_) and \
+            type_.scale is not None and \
+            type_.precision is not None:
+            return self._extend_numeric(type_, 
+                            "FLOAT(%s, %s)" % (type_.precision, type_.scale))
         elif type_.precision is not None:
             return self._extend_numeric(type_, "FLOAT(%s)" % (type_.precision,))
         else:
             return self._extend_numeric(type_, "FLOAT")
-    
+
     def visit_INTEGER(self, type_):
         if self._mysql_type(type_) and type_.display_width is not None:
-            return self._extend_numeric(type_, "INTEGER(%(display_width)s)" % {'display_width': type_.display_width})
+            return self._extend_numeric(type_, 
+                        "INTEGER(%(display_width)s)" % 
+                        {'display_width': type_.display_width})
         else:
             return self._extend_numeric(type_, "INTEGER")
-        
+
     def visit_BIGINT(self, type_):
         if self._mysql_type(type_) and type_.display_width is not None:
-            return self._extend_numeric(type_, "BIGINT(%(display_width)s)" % {'display_width': type_.display_width})
+            return self._extend_numeric(type_, 
+                        "BIGINT(%(display_width)s)" % 
+                        {'display_width': type_.display_width})
         else:
             return self._extend_numeric(type_, "BIGINT")
-    
+
     def visit_MEDIUMINT(self, type_):
         if self._mysql_type(type_) and type_.display_width is not None:
-            return self._extend_numeric(type_, "MEDIUMINT(%(display_width)s)" % {'display_width': type_.display_width})
+            return self._extend_numeric(type_, 
+                        "MEDIUMINT(%(display_width)s)" % 
+                        {'display_width': type_.display_width})
         else:
             return self._extend_numeric(type_, "MEDIUMINT")
 
@@ -1493,7 +1512,10 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
 
     def visit_SMALLINT(self, type_):
         if self._mysql_type(type_) and type_.display_width is not None:
-            return self._extend_numeric(type_, "SMALLINT(%(display_width)s)" % {'display_width': type_.display_width})
+            return self._extend_numeric(type_, 
+                        "SMALLINT(%(display_width)s)" % 
+                        {'display_width': type_.display_width}
+                    )
         else:
             return self._extend_numeric(type_, "SMALLINT")
 
@@ -1502,7 +1524,7 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
             return "BIT(%s)" % type_.length
         else:
             return "BIT"
-    
+
     def visit_DATETIME(self, type_):
         return "DATETIME"
 
@@ -1520,67 +1542,71 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
             return "YEAR"
         else:
             return "YEAR(%s)" % type_.display_width
-    
+
     def visit_TEXT(self, type_):
         if type_.length:
             return self._extend_string(type_, {}, "TEXT(%d)" % type_.length)
         else:
             return self._extend_string(type_, {}, "TEXT")
-        
+
     def visit_TINYTEXT(self, type_):
         return self._extend_string(type_, {}, "TINYTEXT")
 
     def visit_MEDIUMTEXT(self, type_):
         return self._extend_string(type_, {}, "MEDIUMTEXT")
-    
+
     def visit_LONGTEXT(self, type_):
         return self._extend_string(type_, {}, "LONGTEXT")
-    
+
     def visit_VARCHAR(self, type_):
         if type_.length:
             return self._extend_string(type_, {}, "VARCHAR(%d)" % type_.length)
         else:
-            raise exc.InvalidRequestError("VARCHAR requires a length when rendered on MySQL")
-    
+            raise exc.InvalidRequestError(
+                    "VARCHAR requires a length on dialect %s" % 
+                    self.dialect.name)
+
     def visit_CHAR(self, type_):
         if type_.length:
             return self._extend_string(type_, {}, "CHAR(%(length)s)" % {'length' : type_.length})
         else:
             return self._extend_string(type_, {}, "CHAR")
-            
+
     def visit_NVARCHAR(self, type_):
         # We'll actually generate the equiv. "NATIONAL VARCHAR" instead
         # of "NVARCHAR".
         if type_.length:
             return self._extend_string(type_, {'national':True}, "VARCHAR(%(length)s)" % {'length': type_.length})
         else:
-            raise exc.InvalidRequestError("NVARCHAR requires a length when rendered on MySQL")
-    
+            raise exc.InvalidRequestError(
+                    "NVARCHAR requires a length on dialect %s" % 
+                    self.dialect.name)
+
     def visit_NCHAR(self, type_):
         # We'll actually generate the equiv. "NATIONAL CHAR" instead of "NCHAR".
         if type_.length:
             return self._extend_string(type_, {'national':True}, "CHAR(%(length)s)" % {'length': type_.length})
         else:
             return self._extend_string(type_, {'national':True}, "CHAR")
-    
+
     def visit_VARBINARY(self, type_):
         return "VARBINARY(%d)" % type_.length
-    
+
     def visit_large_binary(self, type_):
         return self.visit_BLOB(type_)
-    
+
     def visit_enum(self, type_):
         if not type_.native_enum:
             return super(MySQLTypeCompiler, self).visit_enum(type_)
         else:
             return self.visit_ENUM(type_)
-    
+
     def visit_BLOB(self, type_):
         if type_.length:
             return "BLOB(%d)" % type_.length
         else:
             return "BLOB"
-    
+
     def visit_TINYBLOB(self, type_):
         return "TINYBLOB"
 
@@ -1595,13 +1621,13 @@ class MySQLTypeCompiler(compiler.GenericTypeCompiler):
         for e in type_.enums:
             quoted_enums.append("'%s'" % e.replace("'", "''"))
         return self._extend_string(type_, {}, "ENUM(%s)" % ",".join(quoted_enums))
-        
+
     def visit_SET(self, type_):
         return self._extend_string(type_, {}, "SET(%s)" % ",".join(type_._ddl_values))
 
     def visit_BOOLEAN(self, type):
         return "BOOL"
-        
+
 
 class MySQLIdentifierPreparer(compiler.IdentifierPreparer):
 
@@ -1611,7 +1637,7 @@ class MySQLIdentifierPreparer(compiler.IdentifierPreparer):
         if not server_ansiquotes:
             quote = "`"
         else:
-            quote = '"'    
+            quote = '"'
 
         super(MySQLIdentifierPreparer, self).__init__(
                                                 dialect, 
@@ -1625,32 +1651,34 @@ class MySQLIdentifierPreparer(compiler.IdentifierPreparer):
 
 class MySQLDialect(default.DefaultDialect):
     """Details of the MySQL dialect.  Not used directly in application code."""
-    
+
     name = 'mysql'
     supports_alter = True
+
     # identifiers are 64, however aliases can be 255...
     max_identifier_length = 255
-    
+    max_index_name_length = 64
+
     supports_native_enum = True
-    
+
     supports_sane_rowcount = True
     supports_sane_multi_rowcount = False
-    
+
     default_paramstyle = 'format'
     colspecs = colspecs
-    
+
     statement_compiler = MySQLCompiler
     ddl_compiler = MySQLDDLCompiler
     type_compiler = MySQLTypeCompiler
     ischema_names = ischema_names
     preparer = MySQLIdentifierPreparer
-    
+
     # default SQL compilation settings -
     # these are modified upon initialize(), 
     # i.e. first connect
     _backslash_escapes = True
     _server_ansiquotes = False
-    
+
     def __init__(self, use_ansiquotes=None, **kwargs):
         default.DefaultDialect.__init__(self, **kwargs)
 
@@ -1706,11 +1734,11 @@ class MySQLDialect(default.DefaultDialect):
         resultset = connection.execute("XA RECOVER")
         return [row['data'][0:row['gtrid_length']] for row in resultset]
 
-    def is_disconnect(self, e):
+    def is_disconnect(self, e, connection, cursor):
         if isinstance(e, self.dbapi.OperationalError):
             return self._extract_error_code(e) in \
                         (2006, 2013, 2014, 2045, 2055)
-        elif isinstance(e, self.dbapi.InterfaceError):  
+        elif isinstance(e, self.dbapi.InterfaceError):
             # if underlying connection is closed, 
             # this is the error you get
             return "(0, '')" in str(e)
@@ -1734,7 +1762,7 @@ class MySQLDialect(default.DefaultDialect):
 
     def _extract_error_code(self, exception):
         raise NotImplementedError()
-    
+
     def _get_default_schema_name(self, connection):
         return connection.execute('SELECT DATABASE()').scalar()
 
@@ -1762,14 +1790,14 @@ class MySQLDialect(default.DefaultDialect):
                 have = rs.rowcount > 0
                 rs.close()
                 return have
-            except exc.SQLError, e:
+            except exc.DBAPIError, e:
                 if self._extract_error_code(e.orig) == 1146:
                     return False
                 raise
         finally:
             if rs:
                 rs.close()
-    
+
     def initialize(self, connection):
         default.DefaultDialect.initialize(self, connection)
         self._connection_charset = self._detect_charset(connection)
@@ -1786,7 +1814,7 @@ class MySQLDialect(default.DefaultDialect):
     def _supports_cast(self):
         return self.server_version_info is None or \
                     self.server_version_info >= (4, 0, 2)
-        
+
     @reflection.cache
     def get_schema_names(self, connection, **kw):
         rp = connection.execute("SHOW schemas")
@@ -1811,7 +1839,7 @@ class MySQLDialect(default.DefaultDialect):
 
             return [row[0] for row in self._compat_fetchall(rp, charset=charset)\
                                                         if row[1] == 'BASE TABLE']
-            
+
     @reflection.cache
     def get_view_names(self, connection, schema=None, **kw):
         charset = self._connection_charset
@@ -1853,7 +1881,7 @@ class MySQLDialect(default.DefaultDialect):
 
         parsed_state = self._parsed_state_or_create(connection, table_name, schema, **kw)
         default_schema = None
-        
+
         fkeys = []
 
         for spec in parsed_state.constraints:
@@ -1891,7 +1919,7 @@ class MySQLDialect(default.DefaultDialect):
     def get_indexes(self, connection, table_name, schema=None, **kw):
 
         parsed_state = self._parsed_state_or_create(connection, table_name, schema, **kw)
-        
+
         indexes = []
         for spec in parsed_state.keys:
             unique = False
@@ -1931,14 +1959,14 @@ class MySQLDialect(default.DefaultDialect):
                         schema, 
                         info_cache=kw.get('info_cache', None)
                     )
-    
+
     @util.memoized_property
     def _tabledef_parser(self):
         """return the MySQLTableDefinitionParser, generate if needed.
-        
+
         The deferred creation ensures that the dialect has 
         retrieved server version information first.
-        
+
         """
         if (self.server_version_info < (4, 1) and self._server_ansiquotes):
             # ANSI_QUOTES doesn't affect SHOW CREATE TABLE on < 4.1
@@ -1946,7 +1974,7 @@ class MySQLDialect(default.DefaultDialect):
         else:
             preparer = self.identifier_preparer
         return MySQLTableDefinitionParser(self, preparer)
-        
+
     @reflection.cache
     def _setup_parser(self, connection, table_name, schema=None, **kw):
         charset = self._connection_charset
@@ -1961,17 +1989,6 @@ class MySQLDialect(default.DefaultDialect):
                                            full_name=full_name)
             sql = parser._describe_to_create(table_name, columns)
         return parser.parse(sql, charset)
-  
-    def _adjust_casing(self, table, charset=None):
-        """Adjust Table name to the server case sensitivity, if needed."""
-
-        casing = self._server_casing
-
-        # For winxx database hosts.  TODO: is this really needed?
-        if casing == 1 and table.name != table.name.lower():
-            table.name = table.name.lower()
-            lc_alias = sa_schema._get_table_key(table.name, table.schema)
-            table.metadata.tables[lc_alias] = table
 
     def _detect_charset(self, connection):
         raise NotImplementedError()
@@ -2035,10 +2052,10 @@ class MySQLDialect(default.DefaultDialect):
                 mode = (mode_no | 4 == mode_no) and 'ANSI_QUOTES' or ''
 
         self._server_ansiquotes = 'ANSI_QUOTES' in mode
-        
+
         # as of MySQL 5.0.1
         self._backslash_escapes = 'NO_BACKSLASH_ESCAPES' not in mode
-        
+
     def _show_create_table(self, connection, table, charset=None,
                            full_name=None):
         """Run SHOW CREATE TABLE for a ``Table``."""
@@ -2050,7 +2067,7 @@ class MySQLDialect(default.DefaultDialect):
         rp = None
         try:
             rp = connection.execute(st)
-        except exc.SQLError, e:
+        except exc.DBAPIError, e:
             if self._extract_error_code(e.orig) == 1146:
                 raise exc.NoSuchTableError(full_name)
             else:
@@ -2074,7 +2091,7 @@ class MySQLDialect(default.DefaultDialect):
         try:
             try:
                 rp = connection.execute(st)
-            except exc.SQLError, e:
+            except exc.DBAPIError, e:
                 if self._extract_error_code(e.orig) == 1146:
                     raise exc.NoSuchTableError(full_name)
                 else:
@@ -2087,17 +2104,17 @@ class MySQLDialect(default.DefaultDialect):
 
 class ReflectedState(object):
     """Stores raw information about a SHOW CREATE TABLE statement."""
-    
+
     def __init__(self):
         self.columns = []
         self.table_options = {}
         self.table_name = None
         self.keys = []
         self.constraints = []
-        
+
 class MySQLTableDefinitionParser(object):
     """Parses the results of a SHOW CREATE TABLE statement."""
-    
+
     def __init__(self, dialect, preparer):
         self.dialect = dialect
         self.preparer = preparer
@@ -2130,9 +2147,9 @@ class MySQLTableDefinitionParser(object):
                     state.constraints.append(spec)
                 else:
                     pass
-                    
+
         return state
-        
+
     def _parse_constraints(self, line):
         """Parse a KEY or CONSTRAINT line.
 
@@ -2206,7 +2223,7 @@ class MySQLTableDefinitionParser(object):
             options.pop(nope, None)
 
         for opt, val in options.items():
-            state.table_options['mysql_%s' % opt] = val
+            state.table_options['%s_%s' % (self.dialect.name, opt)] = val
 
     def _parse_column(self, line, state):
         """Extract column details.
@@ -2283,7 +2300,7 @@ class MySQLTableDefinitionParser(object):
         if default == 'NULL':
             # eliminates the need to deal with this later.
             default = None
-            
+
         col_d = dict(name=name, type=type_instance, default=default)
         col_d.update(col_kw)
         state.columns.append(col_d)
@@ -2390,8 +2407,8 @@ class MySQLTableDefinitionParser(object):
             r'(?: +COLLATE +(?P<collate>[\w_]+))?'
             r'(?: +(?P<notnull>NOT NULL))?'
             r'(?: +DEFAULT +(?P<default>'
-              r'(?:NULL|\x27(?:\x27\x27|[^\x27])*\x27|\w+)'
-              r'(?:ON UPDATE \w+)?'
+              r'(?:NULL|\x27(?:\x27\x27|[^\x27])*\x27|\w+'
+              r'(?: +ON UPDATE \w+)?)'
             r'))?'
             r'(?: +(?P<autoincr>AUTO_INCREMENT))?'
             r'(?: +COMMENT +(P<comment>(?:\x27\x27|[^\x27])+))?'
