@@ -1,3 +1,9 @@
+# sql/__init__.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 from sqlalchemy.sql.expression import (
     Alias,
     ClauseElement,
@@ -42,11 +48,13 @@ from sqlalchemy.sql.expression import (
     or_,
     outerjoin,
     outparam,
+    over,
     select,
     subquery,
     table,
     text,
     tuple_,
+    type_coerce,
     union,
     union_all,
     update,
