@@ -1,4 +1,11 @@
-from sqlalchemy import exc, schema, topological, util, sql, types as sqltypes
+# sql/util.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
+from sqlalchemy import exc, schema, util, sql, types as sqltypes
+from sqlalchemy.util import topological
 from sqlalchemy.sql import expression, operators, visitors
 from itertools import chain
 
@@ -6,7 +13,7 @@ from itertools import chain
 
 def sort_tables(tables):
     """sort a collection of Table objects in order of their foreign-key dependency."""
-    
+
     tables = list(tables)
     tuples = []
     def visit_foreign_key(fkey):
@@ -26,7 +33,7 @@ def sort_tables(tables):
         tuples.extend(
             [parent, table] for parent in table._extra_dependencies
         )
-                            
+
     return list(topological.sort(tuples, tables))
 
 def find_join_source(clauses, join_to):
@@ -34,18 +41,18 @@ def find_join_source(clauses, join_to):
     return the first index and element from the list of 
     clauses which can be joined against the selectable.  returns 
     None, None if no match is found.
-    
+
     e.g.::
-    
+
         clause1 = table1.join(table2)
         clause2 = table4.join(table5)
-        
+
         join_to = table2.join(table3)
-        
+
         find_join_source([clause1, clause2], join_to) == clause1
-    
+
     """
-    
+
     selectables = list(expression._from_objects(join_to))
     for i, f in enumerate(clauses):
         for s in selectables:
@@ -58,23 +65,23 @@ def find_tables(clause, check_columns=False,
                 include_aliases=False, include_joins=False, 
                 include_selects=False, include_crud=False):
     """locate Table objects within the given expression."""
-    
+
     tables = []
     _visitors = {}
-    
+
     if include_selects:
         _visitors['select'] = _visitors['compound_select'] = tables.append
-    
+
     if include_joins:
         _visitors['join'] = tables.append
-        
+
     if include_aliases:
         _visitors['alias']  = tables.append
-    
+
     if include_crud:
         _visitors['insert'] = _visitors['update'] = \
                     _visitors['delete'] = lambda ent: tables.append(ent.table)
-        
+
     if check_columns:
         def visit_column(column):
             tables.append(column.table)
@@ -87,26 +94,70 @@ def find_tables(clause, check_columns=False,
 
 def find_columns(clause):
     """locate Column objects within the given expression."""
-    
+
     cols = util.column_set()
     visitors.traverse(clause, {}, {'column':cols.add})
     return cols
 
+def clause_is_present(clause, search):
+    """Given a target clause and a second to search within, return True
+    if the target is plainly present in the search without any
+    subqueries or aliases involved.
+
+    Basically descends through Joins.
+
+    """
+
+    stack = [search]
+    while stack:
+        elem = stack.pop()
+        if clause is elem:
+            return True
+        elif isinstance(elem, expression.Join):
+            stack.extend((elem.left, elem.right))
+    return False
+
+
+def bind_values(clause):
+    """Return an ordered list of "bound" values in the given clause.
+
+    E.g.::
+
+        >>> expr = and_(
+        ...    table.c.foo==5, table.c.foo==7
+        ... )
+        >>> bind_values(expr)
+        [5, 7]
+    """
+
+    v = []
+    def visit_bindparam(bind):
+        value = bind.value
+
+        # evaluate callables
+        if callable(value):
+            value = value()
+
+        v.append(value)
+
+    visitors.traverse(clause, {}, {'bindparam':visit_bindparam})
+    return v
+
 def _quote_ddl_expr(element):
     if isinstance(element, basestring):
         element = element.replace("'", "''")
         return "'%s'" % element
     else:
         return repr(element)
-    
+
 def expression_as_ddl(clause):
     """Given a SQL expression, convert for usage in DDL, such as 
      CREATE INDEX and CHECK CONSTRAINT.
-     
+
      Converts bind params into quoted literals, column identifiers
      into detached column constructs so that the parent table
      identifier is not included.
-    
+
     """
     def repl(element):
         if isinstance(element, expression._BindParamClause):
@@ -116,9 +167,9 @@ def expression_as_ddl(clause):
             return expression.column(element.name)
         else:
             return None
-        
+
     return visitors.replacement_traverse(clause, {}, repl)
-    
+
 def adapt_criterion_to_null(crit, nulls):
     """given criterion containing bind params, convert selected elements to IS NULL."""
 
@@ -135,28 +186,25 @@ def adapt_criterion_to_null(crit, nulls):
             binary.negate = operators.isnot
 
     return visitors.cloned_traverse(crit, {}, {'binary':visit_binary})
-    
-    
+
 def join_condition(a, b, ignore_nonexistent_tables=False, a_subset=None):
     """create a join condition between two tables or selectables.
-    
+
     e.g.::
-    
+
         join_condition(tablea, tableb)
-        
+
     would produce an expression along the lines of::
-    
+
         tablea.c.id==tableb.c.tablea_id
-    
+
     The join is determined based on the foreign key relationships
     between the two selectables.   If there are multiple ways
     to join, or no way to join, an error is raised.
-    
-    :param ignore_nonexistent_tables: This flag will cause the
-    function to silently skip over foreign key resolution errors
-    due to nonexistent tables - the assumption is that these
-    tables have not yet been defined within an initialization process
-    and are not significant to the operation.
+
+    :param ignore_nonexistent_tables:  Deprecated - this
+    flag is no longer used.  Only resolution errors regarding
+    the two given tables are propagated.
 
     :param a_subset: An optional expression that is a sub-component
     of ``a``.  An attempt will be made to join to just this sub-component
@@ -164,45 +212,52 @@ def join_condition(a, b, ignore_nonexistent_tables=False, a_subset=None):
     will be successful even if there are other ways to join to ``a``.
     This allows the "right side" of a join to be passed thereby
     providing a "natural join".
-    
+
     """
     crit = []
     constraints = set()
-    
+
     for left in (a_subset, a):
         if left is None:
             continue
-        for fk in b.foreign_keys:
+        for fk in sorted(
+                    b.foreign_keys, 
+                    key=lambda fk:fk.parent._creation_order):
             try:
                 col = fk.get_referent(left)
-            except exc.NoReferencedTableError:
-                if ignore_nonexistent_tables:
-                    continue
-                else:
+            except exc.NoReferenceError, nrte:
+                if nrte.table_name == left.name:
                     raise
-                
+                else:
+                    continue
+
             if col is not None:
                 crit.append(col == fk.parent)
                 constraints.add(fk.constraint)
         if left is not b:
-            for fk in left.foreign_keys:
+            for fk in sorted(
+                        left.foreign_keys, 
+                        key=lambda fk:fk.parent._creation_order):
                 try:
                     col = fk.get_referent(b)
-                except exc.NoReferencedTableError:
-                    if ignore_nonexistent_tables:
-                        continue
-                    else:
+                except exc.NoReferenceError, nrte:
+                    if nrte.table_name == b.name:
                         raise
+                    else:
+                        # this is totally covered.  can't get
+                        # coverage to mark it.
+                        continue
 
                 if col is not None:
                     crit.append(col == fk.parent)
                     constraints.add(fk.constraint)
         if crit:
             break
-            
+
     if len(crit) == 0:
         if isinstance(b, expression._FromGrouping):
-            hint = " Perhaps you meant to convert the right side to a subquery using alias()?"
+            hint = " Perhaps you meant to convert the right side to a "\
+                                "subquery using alias()?"
         else:
             hint = ""
         raise exc.ArgumentError(
@@ -223,17 +278,17 @@ def join_condition(a, b, ignore_nonexistent_tables=False, a_subset=None):
 
 class Annotated(object):
     """clones a ClauseElement and applies an 'annotations' dictionary.
-    
+
     Unlike regular clones, this clone also mimics __hash__() and 
     __cmp__() of the original element so that it takes its place
     in hashed collections.
-    
+
     A reference to the original element is maintained, for the important
     reason of keeping its hash value current.  When GC'ed, the 
     hash value may be reused, causing conflicts.
 
     """
-    
+
     def __new__(cls, *args):
         if not args:
             # clone constructor
@@ -255,11 +310,11 @@ class Annotated(object):
         # collections into __dict__
         if isinstance(element, expression.FromClause):
             element.c
-        
+
         self.__dict__ = element.__dict__.copy()
         self.__element = element
         self._annotations = values
-        
+
     def _annotate(self, values):
         _values = self._annotations.copy()
         _values.update(values)
@@ -267,28 +322,28 @@ class Annotated(object):
         clone.__dict__ = self.__dict__.copy()
         clone._annotations = _values
         return clone
-    
+
     def _deannotate(self):
         return self.__element
-    
+
     def _compiler_dispatch(self, visitor, **kw):
         return self.__element.__class__._compiler_dispatch(self, visitor, **kw)
-        
+
     @property
     def _constructor(self):
         return self.__element._constructor
-        
+
     def _clone(self):
         clone = self.__element._clone()
         if clone is self.__element:
             # detect immutable, don't change anything
             return self
         else:
-            # update the clone with any changes that have occured
+            # update the clone with any changes that have occurred
             # to this object's __dict__.
             clone.__dict__.update(self.__dict__)
             return Annotated(clone, self._annotations)
-    
+
     def __hash__(self):
         return hash(self.__element)
 
@@ -344,7 +399,7 @@ def _deep_deannotate(element):
 def splice_joins(left, right, stop_on=None):
     if left is None:
         return right
-        
+
     stack = [(right, None)]
 
     adapter = ClauseAdapter(left)
@@ -364,7 +419,7 @@ def splice_joins(left, right, stop_on=None):
             ret = right
 
     return ret
-    
+
 def reduce_columns(columns, *clauses, **kw):
     """given a list of columns, return a 'reduced' set based on natural equivalents.
 
@@ -377,14 +432,14 @@ def reduce_columns(columns, *clauses, **kw):
 
     \**kw may specify 'ignore_nonexistent_tables' to ignore foreign keys
     whose tables are not yet configured.
-    
+
     This function is primarily used to determine the most minimal "primary key"
     from a selectable, by reducing the set of primary key columns present
     in the the selectable to just those that are not repeated.
 
     """
     ignore_nonexistent_tables = kw.pop('ignore_nonexistent_tables', False)
-    
+
     columns = util.ordered_column_set(columns)
 
     omit = util.column_set()
@@ -421,12 +476,12 @@ def reduce_columns(columns, *clauses, **kw):
 def criterion_as_pairs(expression, consider_as_foreign_keys=None, 
                         consider_as_referenced_keys=None, any_operator=False):
     """traverse an expression and locate binary criterion pairs."""
-    
+
     if consider_as_foreign_keys and consider_as_referenced_keys:
         raise exc.ArgumentError("Can only specify one of "
                                 "'consider_as_foreign_keys' or "
                                 "'consider_as_referenced_keys'")
-        
+
     def visit_binary(binary):
         if not any_operator and binary.operator is not operators.eq:
             return
@@ -465,14 +520,14 @@ def criterion_as_pairs(expression, consider_as_foreign_keys=None,
 
 def folded_equivalents(join, equivs=None):
     """Return a list of uniquely named columns.
-    
+
     The column list of the given Join will be narrowed 
     down to a list of all equivalently-named,
     equated columns folded into one column, where 'equated' means they are
     equated to each other in the ON clause of this join.
 
     This function is used by Join.select(fold_equivalents=True).
-    
+
     Deprecated.   This function is used for a certain kind of 
     "polymorphic_union" which is designed to achieve joined
     table inheritance where the base table has no "discriminator"
@@ -508,10 +563,10 @@ def folded_equivalents(join, equivs=None):
 
 class AliasedRow(object):
     """Wrap a RowProxy with a translation map.
-    
+
     This object allows a set of keys to be translated
     to those present in a RowProxy.
-    
+
     """
     def __init__(self, row, map):
         # AliasedRow objects don't nest, so un-nest
@@ -521,7 +576,7 @@ class AliasedRow(object):
         else:
             self.row = row
         self.map = map
-        
+
     def __contains__(self, key):
         return self.map[key] in self.row
 
@@ -537,7 +592,7 @@ class AliasedRow(object):
 
 class ClauseAdapter(visitors.ReplacingCloningVisitor):
     """Clones and modifies clauses based on column correspondence.
-    
+
     E.g.::
 
       table1 = Table('sometable', metadata,
@@ -567,7 +622,7 @@ class ClauseAdapter(visitors.ReplacingCloningVisitor):
         self.include = include
         self.exclude = exclude
         self.equivalents = util.column_dict(equivalents or {})
-        
+
     def _corresponding_column(self, col, require_embedded, _seen=util.EMPTY_SET):
         newcol = self.selectable.corresponding_column(col, require_embedded=require_embedded)
 
@@ -590,17 +645,17 @@ class ClauseAdapter(visitors.ReplacingCloningVisitor):
             return None
         elif self.exclude and col in self.exclude:
             return None
-        
+
         return self._corresponding_column(col, True)
 
 class ColumnAdapter(ClauseAdapter):
     """Extends ClauseAdapter with extra utility functions.
-    
+
     Provides the ability to "wrap" this ClauseAdapter 
     around another, a columns dictionary which returns
     adapted elements given an original, and an 
     adapted_row() factory.
-    
+
     """
     def __init__(self, selectable, equivalents=None, 
                         chain_to=None, include=None, 
@@ -633,11 +688,11 @@ class ColumnAdapter(ClauseAdapter):
         c = self._corresponding_column(col, True)
         if c is None:
             c = self.adapt_clause(col)
-            
+
             # anonymize labels in case they have a hardcoded name
             if isinstance(c, expression._Label):
                 c = c.label(None)
-                
+
         # adapt_required indicates that if we got the same column
         # back which we put in (i.e. it passed through), 
         # it's not correct.  this is used by eagerloading which
@@ -646,17 +701,17 @@ class ColumnAdapter(ClauseAdapter):
         # the wrong column.
         if self.adapt_required and c is col:
             return None
-            
-        return c    
+
+        return c
 
     def adapted_row(self, row):
         return AliasedRow(row, self.columns)
-    
+
     def __getstate__(self):
         d = self.__dict__.copy()
         del d['columns']
         return d
-        
+
     def __setstate__(self, state):
         self.__dict__.update(state)
         self.columns = util.PopulateDict(self._locate_col)
